@@ -17,11 +17,14 @@ from __future__ import unicode_literals
 
 import json
 import logging
+import time
+
 from requests.adapters import HTTPAdapter
+from requests import exceptions as req_exceptions
 from requests import Session
-import six
 
 import acos_client
+from acos_client import errors as ae
 from acos_client import logutils
 from acos_client.v30 import responses as acos_responses
 
@@ -33,12 +36,14 @@ broken_replies = {
 
 
 class HttpClient(object):
+    AXAPI_DEFAULT_REQ_TIMEOUT = 300
     HEADERS = {
         "Content-type": "application/json",
         "User-Agent": "ACOS-Client-AGENT-%s" % acos_client.VERSION,
     }
 
-    def __init__(self, host, port=None, protocol="https", max_retries=3, timeout=5):
+    def __init__(self, host, port=None, protocol="https", max_retries=3,
+                 timeout=AXAPI_DEFAULT_REQ_TIMEOUT):
         if port is None:
             if protocol is 'http':
                 self.port = 80
@@ -51,8 +56,23 @@ class HttpClient(object):
         self.max_retries = max_retries
         self.timeout = timeout
 
-    def request(self, method, api_url, params={}, headers=None,
-                file_name=None, file_content=None, axapi_args=None, **kwargs):
+    def dict_underscore_to_dash(self, my_dict):
+        if type(my_dict) is list:
+            item_list = []
+            for item in my_dict:
+                item_list.append(self.dict_underscore_to_dash(item))
+            return item_list
+        elif type(my_dict) is dict:
+            item_dict = {}
+            for k, v in my_dict.items():
+                item_dict[k.replace('_', '-')] = self.dict_underscore_to_dash(v)
+            return item_dict
+        else:
+            return my_dict
+
+    def request_impl(self, method, api_url, params={}, headers=None,
+                     file_name=None, file_content=None, axapi_args=None,
+                     max_retries=None, timeout=None, **kwargs):
         LOG.debug("axapi_http: full url = %s", self.url_base + api_url)
         LOG.debug("axapi_http: %s url = %s", method, api_url)
         LOG.debug("axapi_http: params = %s", json.dumps(logutils.clean(params), indent=4))
@@ -61,11 +81,10 @@ class HttpClient(object):
 
         # Update params with axapi_args for currently unsupported configuration of objects
         if axapi_args is not None:
-            formatted_axapi_args = dict(
-                [(k.replace('_', '-'), v) for k, v in six.iteritems(axapi_args)]
-            )
+            formatted_axapi_args = self.dict_underscore_to_dash(axapi_args)
             params = acos_client.v21.axapi_http.merge_dicts(params, formatted_axapi_args)
 
+        LOG.debug("axapi_http: params + axapi_args = %s", json.dumps(logutils.clean(params), indent=4))
         # Set data" variable for the request
         if params:
             params_copy = params.copy()
@@ -78,8 +97,10 @@ class HttpClient(object):
            (file_name is not None and file_content is None):
             raise ValueError("file_name and file_content must both be populated if one is")
 
-        max_retries = kwargs.get('max_retries', self.max_retries)
-        timeout = kwargs.get('timeout', self.timeout)
+        if not max_retries:
+            max_retries = self.max_retries
+        if not timeout:
+            timeout = self.timeout
 
         # Set "headers" variable for the request
         request_headers = self.HEADERS.copy()
@@ -142,14 +163,42 @@ class HttpClient(object):
 
         return json_response
 
-    def get(self, api_url, params={}, headers=None, **kwargs):
-        return self.request("GET", api_url, params, headers, **kwargs)
+    def request(self, method, api_url, params={}, headers=None,
+                file_name=None, file_content=None, axapi_args=None,
+                max_retries=None, timeout=None, **kwargs):
+        attemps = 300
+        if timeout and timeout > attemps:
+            attemps = timeout
 
-    def post(self, api_url, params={}, headers=None, **kwargs):
-        return self.request("POST", api_url, params, headers, **kwargs)
+        while attemps > 0:
+            try:
+                return self.request_impl(method, api_url, params, headers,
+                                         file_name=file_name, file_content=file_content,
+                                         max_retries=max_retries,
+                                         timeout=timeout, axapi_args=axapi_args,
+                                         **kwargs)
 
-    def put(self, api_url, params={}, headers=None, **kwargs):
-        return self.request("PUT", api_url, params, headers, **kwargs)
+            except (ae.ACOSSystemIsBusy, ae.ACOSSystemNotReady, req_exceptions.ConnectionError) as e:
+                LOG.warning("ACOS device system is busy: %s", str(e))
+                attemps = attemps - 2
+                if attemps <= 0:
+                    raise e
+                time.sleep(2)
+            except (Exception) as e:
+                raise e
 
-    def delete(self, api_url, params={}, headers=None, **kwargs):
-        return self.request("DELETE", api_url, params, headers, **kwargs)
+    def get(self, api_url, params={}, headers=None, max_retries=None, timeout=None, axapi_args=None, **kwargs):
+        return self.request("GET", api_url, params, headers, max_retries=max_retries,
+                            timeout=timeout, axapi_args=axapi_args, **kwargs)
+
+    def post(self, api_url, params={}, headers=None, max_retries=None, timeout=None, axapi_args=None, **kwargs):
+        return self.request("POST", api_url, params, headers, max_retries=max_retries,
+                            timeout=timeout, axapi_args=axapi_args, **kwargs)
+
+    def put(self, api_url, params={}, headers=None, max_retries=None, timeout=None, axapi_args=None, **kwargs):
+        return self.request("PUT", api_url, params, headers, max_retries=max_retries,
+                            timeout=timeout, axapi_args=axapi_args, **kwargs)
+
+    def delete(self, api_url, params={}, headers=None, max_retries=None, timeout=None, axapi_args=None, **kwargs):
+        return self.request("DELETE", api_url, params, headers, max_retries=max_retries,
+                            timeout=timeout, axapi_args=axapi_args, **kwargs)
